/******************************************************************************
 *
 * The SoundStretch main routine. This program processes WAV audio files by
 * using the SoundTouch library routines.
 *
 * Author        : Copyright (c) Olli Parviainen 2002
 * Author e-mail : oparviai@iki.fi
 * File created  : 13-Jan-2002
 * Last modified : 21-Feb-2002
 *
 * Change History :
 *
 * 21-Feb-2002 OP : Changed version number to v1.01 due to changes in
 *                  WavFile.cpp
 *
 * License :
 *
 *  This file is part of SoundTouch sound processing library.
 *
 *  SoundTouch is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  SoundTouch is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SoundTouch; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <stdexcept>
#include <stdio.h>
#include "RunParameters.h"
#include "WavFile.h"
#include "SoundTouch.h"

using namespace std;

#define BUFF_SIZE       2048
#define BUFF_SIZE_BYTES (2 * BUFF_SIZE)

static const char helloText[] = 
    "\n"
    "   SoundStretch v1.01 -  Written by Olli Parviainen 2001 - 2002\n"
    "==================================================================\n"
    "author e-mail: <oparviai@iki.fi> - WWW: http://www.iki.fi/oparviai\n"
    "\n";

static void openFiles(WavInFile **inFile, WavOutFile **outFile, const RunParameters *params)
{
    int bits, samplerate, channels;

    // open input file...
    *inFile = new WavInFile(params->inFileName);

    // ... open output file with same sound parameters
    bits = (*inFile)->getNumBits();
    samplerate = (*inFile)->getSampleRate();
    channels = (*inFile)->getNumChannels();

    *outFile = new WavOutFile(params->outFileName, samplerate, bits, channels);
}



// Sets the 'SoundTouch' object up according to input file sound format & 
// command line parameters
static void setup(SoundTouch *pSoundTouch, const WavInFile *inFile, const RunParameters *params)
{
    int sampleRate;
    int channels;

    sampleRate = inFile->getSampleRate();
    channels = inFile->getNumChannels();
    pSoundTouch->setSampleRate(sampleRate);
    pSoundTouch->setChannels(channels);

    pSoundTouch->setTempoChange(params->tempoDelta);
    pSoundTouch->setPitchSemiTones(params->pitchDelta);
    pSoundTouch->setRateChange(params->rateDelta);

    pSoundTouch->setSetting(SETTING_USE_QUICKSEEK, params->quick);
    pSoundTouch->setSetting(SETTING_USE_AA_FILTER, !params->noAntiAlias);

    // print processing information
    printf("Processing the file with the following changes:\n");
    printf("  tempo change = %+g %%\n", params->tempoDelta);
    printf("  pitch change = %+g semitones\n", params->pitchDelta);
    printf("  rate change  = %+g %%\n\n", params->rateDelta);
    printf("Working...");
}



// Processes the sound: Read sound data from the input file ('inFile'), 
// process with the processing object ('pSoundTouch') and write to the 
// output file ('outFile').
static void process(SoundTouch *pSoundTouch, WavInFile *inFile, WavOutFile *outFile)
{
    int nBytes;
    int nSamples;
    int bytesPerSample;
    int buffSizeSamples;
    short buffer[BUFF_SIZE];

    bytesPerSample = inFile->getBytesPerSample();
    buffSizeSamples = BUFF_SIZE_BYTES / bytesPerSample;
    // Process the sound file: Iterate in a loop until end of the input file 
    // is reached.
    do {
        // Read data from the input file into 'buffer'
        nBytes = inFile->read(buffer, BUFF_SIZE_BYTES);
        // Calculate number of samples read
        nSamples = nBytes / bytesPerSample;
        // Feed the sound data into the processing object
        pSoundTouch->putSamples(buffer, nSamples);

        // Receive processed data from the processing object and write into
        // the output file. Iterate this in loop until all the available sound
        // data is extracted from the processing object, for the case that the
        // processing object could output more samples at a time than fits into
        // 'buffer'.
        do {
            // receive processed audio data
            nSamples = pSoundTouch->receiveSamples(buffer, buffSizeSamples);
            // calculate processed audio data size in bytes
            nBytes = nSamples * bytesPerSample;
            assert(nBytes <= BUFF_SIZE_BYTES);
            // write the sound data into the output file.
            outFile->write(buffer, nBytes);
        } while (nSamples != 0);
    } while (inFile->eof() == 0);

    // Extract few last samples from the processing pipeline.
    // First call the 'flush' function.
    pSoundTouch->flush();
    // ...then iterate the receiveSamples loop until all available data from
    // the processing object is extracted and written to the output file.
    do {
        nSamples = pSoundTouch->receiveSamples(buffer, buffSizeSamples);
        nBytes = nSamples * bytesPerSample;
        outFile->write(buffer, nBytes);
    } while (nSamples != 0);
}



int main(const int nParams, const char *paramStr[])
{
    WavInFile *inFile;
    WavOutFile *outFile;
    SoundTouch SoundTouch;
    RunParameters *params;

    printf(helloText);
    try {
        // Parse command line parameters
        params = new RunParameters(nParams, paramStr);

        // Open input & output files
        openFiles(&inFile, &outFile, params);

        // Setup the 'SoundTouch' object for processing the sound
        setup(&SoundTouch, inFile, params);

        // Process the sound
        process(&SoundTouch, inFile, outFile);

        // Close WAV file handles & dispose of the objects
        delete inFile;
        delete outFile;
        delete params;

        printf("Done!\n");
    } catch (runtime_error &e) {
        // An exception occurred during processing, display an error message
        printf("%s\n", e.what());
        return -1;
    }

    return 0;
}
